"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.behaviorpack_check_blockid = exports.behaviorpack_check_blockdescriptor = void 0;
const diagnose_1 = require("../BlockState/diagnose");
const Definitions_1 = require("../../Definitions");
const Types_1 = require("../../../Types");
const bc_minecraft_bedrock_types_1 = require("bc-minecraft-bedrock-types");
const bc_minecraft_bedrock_vanilla_data_1 = require("bc-minecraft-bedrock-vanilla-data");
/**
 *
 * @param blockDescriptor
 * @param diagnoser
 */
function behaviorpack_check_blockdescriptor(blockDescriptor, diagnoser) {
    behaviorpack_check_blockid(blockDescriptor, diagnoser);
    (0, diagnose_1.behaviorpack_check_blockstates)(blockDescriptor, diagnoser);
}
exports.behaviorpack_check_blockdescriptor = behaviorpack_check_blockdescriptor;
/**Checks if the blocks exists in the project or in vanilla, if not then a bug is reported
 * @param id
 * @param diagnoser
 * @returns
 */
function behaviorpack_check_blockid(blockDescriptor, diagnoser) {
    let id = bc_minecraft_bedrock_types_1.Minecraft.Block.getId(blockDescriptor.text);
    //Defined in McProject
    if ((0, Definitions_1.check_definition_value)(diagnoser.project.definitions.block, id, diagnoser))
        return true;
    const data = diagnoser.context.getCache();
    //Project has block
    if (data.hasBlock(id))
        return true;
    const edu = (0, Definitions_1.education_enabled)(diagnoser);
    //Vanilla has block
    if (bc_minecraft_bedrock_vanilla_data_1.MinecraftData.BehaviorPack.hasBlock(id, edu))
        return true;
    //Missing namespace?
    if (!id.includes(":")) {
        //retry
        id = "minecraft:" + id;
        //Defined in McProject
        if ((0, Definitions_1.check_definition_value)(diagnoser.project.definitions.block, id, diagnoser))
            return true;
        //Project has block
        if (data.hasBlock(id))
            return true;
    }
    //Vanilla has block
    if (bc_minecraft_bedrock_vanilla_data_1.MinecraftData.BehaviorPack.hasBlock(id, edu))
        return true;
    //Nothing then report error
    diagnoser.add(id, `Cannot find block definition: ${id}`, Types_1.DiagnosticSeverity.error, "behaviorpack.block.missing");
    return false;
}
exports.behaviorpack_check_blockid = behaviorpack_check_blockid;
//# sourceMappingURL=diagnose.js.map