"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Diagnose = void 0;
const anim_or_controller_1 = require("../anim or controller");
const usage_1 = require("../Animation/usage");
const events_1 = require("./events");
const components_1 = require("./components");
const dependencies_1 = require("./components/dependencies");
const bc_minecraft_molang_1 = require("bc-minecraft-molang");
const diagnostics_1 = require("../../Molang/diagnostics");
const Script_1 = require("../../Minecraft/Script");
const components_2 = require("bc-minecraft-bedrock-types/lib/src/minecraft/components");
const bc_minecraft_bedrock_project_1 = require("bc-minecraft-bedrock-project");
const Json_1 = require("../../Json/Json");
const bc_minecraft_bedrock_types_1 = require("bc-minecraft-bedrock-types");
const properties_1 = require("./properties");
/**Diagnoses the given document as an bp entity
 * @param doc The text document to diagnose
 * @param diagnoser The diagnoser builder to receive the errors*/
function Diagnose(diagnoser) {
    var _a, _b;
    //Check molang
    (0, diagnostics_1.diagnose_molang)(diagnoser.document.getText(), "Entities", diagnoser);
    const entity = Json_1.Json.LoadReport(diagnoser);
    if (!bc_minecraft_bedrock_project_1.Internal.BehaviorPack.Entity.is(entity))
        return;
    //No resource-pack check, entities can exist without their rp side
    //check components
    const context = {
        components: (0, components_2.getUsedComponents)(entity["minecraft:entity"]),
    };
    (0, dependencies_1.behaviorpack_entity_components_dependencies)(entity, context, diagnoser);
    (0, components_1.behaviorpack_entity_components_check)(entity, context, diagnoser);
    const container = entity["minecraft:entity"];
    const MolangData = bc_minecraft_molang_1.Molang.MolangFullSet.harvest(container);
    const id = container.description.identifier;
    const owner = {
        id: id,
        molang: MolangData,
        animations: bc_minecraft_molang_1.DefinedUsing.create(),
    };
    const properties = (_b = Object.entries((_a = container.description.properties) !== null && _a !== void 0 ? _a : {})) === null || _b === void 0 ? void 0 : _b.map(([name, value]) => {
        return { name, ...value };
    });
    //Convert animations / controllers
    bc_minecraft_bedrock_types_1.Types.Definition.forEach(container.description.animations, (ref, anim_id) => {
        owner.animations.defined.push(ref);
        owner.animations.using.push(anim_id);
    });
    //Check animations / animation controllers implements
    owner.animations.using.forEach((anim_id) => (0, anim_or_controller_1.animation_or_controller_diagnose_implementation)(anim_id, owner, "Entities", diagnoser));
    //Script check
    if (container.description.scripts)
        (0, Script_1.diagnose_script)(diagnoser, container.description.scripts, container.description.animations);
    //Check used animations
    (0, usage_1.behaviorpack_animation_used)(container.description.animations, diagnoser, container.description.scripts);
    (0, properties_1.diagnose_entity_properties_definition)(properties, diagnoser);
    //Check events
    if (container.events)
        (0, events_1.behaviorpack_entity_check_events)(container.events, diagnoser, properties, container.component_groups);
}
exports.Diagnose = Diagnose;
//# sourceMappingURL=entry.js.map