"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.behaviorpack_entity_check_event = exports.behaviorpack_entity_check_events = void 0;
const bc_minecraft_bedrock_project_1 = require("bc-minecraft-bedrock-project");
const main_1 = require("../../../../main");
const filters_1 = require("./components/filters");
const properties_1 = require("./properties");
const Mcfunction_1 = require("../Mcfunction");
function behaviorpack_entity_check_events(events, diagnoser, properties, component_groups) {
    if (Array.isArray(events)) {
        events.forEach((event) => behaviorpack_entity_check_event(event, "", diagnoser, properties, component_groups));
    }
    else {
        bc_minecraft_bedrock_project_1.SMap.forEach(events, (event, key) => behaviorpack_entity_check_event(event, key, diagnoser, properties, component_groups));
    }
}
exports.behaviorpack_entity_check_events = behaviorpack_entity_check_events;
/**
 *
 * @param event
 * @param diagnoser
 * @param component_groups
 */
function behaviorpack_entity_check_event(event, event_id, diagnoser, properties, component_groups) {
    var _a, _b, _c, _d;
    has_groups(diagnoser, event_id, (_a = event.add) === null || _a === void 0 ? void 0 : _a.component_groups, component_groups);
    has_groups(diagnoser, event_id, (_b = event.remove) === null || _b === void 0 ? void 0 : _b.component_groups, component_groups);
    (_c = event.randomize) === null || _c === void 0 ? void 0 : _c.forEach((item) => {
        behaviorpack_entity_check_event(item, event_id, diagnoser, properties, component_groups);
    });
    (_d = event.sequence) === null || _d === void 0 ? void 0 : _d.forEach((item) => {
        behaviorpack_entity_check_event(item, event_id, diagnoser, properties, component_groups);
    });
    (0, filters_1.behaviorpack_entity_components_filters)(event, diagnoser);
    if (event.set_property) {
        for (const [key, value] of Object.entries(event.set_property)) {
            (0, properties_1.diagnose_entity_property_usage)(properties, key, value, "events", diagnoser);
        }
    }
    if (event["run_command"]) {
        diagnoser.add(`events/${event_id}`, `Event is using the deprecated run_command property, use queue_command instead`, main_1.DiagnosticSeverity.warning, "behaviorpack.entity.event.run_command");
    }
    if (event.queue_command) {
        const c = event.queue_command.command;
        const command = typeof c === "string" ? [c] : c;
        command.forEach((cmd) => {
            if (cmd.startsWith("/")) {
                diagnoser.add(`events/${event_id}/cmd`, `Commands in queue_command should not start with a /, remove it`, main_1.DiagnosticSeverity.warning, "behaviorpack.entity.event.queue_command");
                cmd = cmd.slice(1);
            }
            ;
            (0, Mcfunction_1.commandsCheck)(cmd, diagnoser);
        });
    }
}
exports.behaviorpack_entity_check_event = behaviorpack_entity_check_event;
function has_groups(diagnoser, id, groups, component_groups) {
    if (groups === undefined)
        return;
    for (let I = 0; I < groups.length; I++) {
        const group = groups[I];
        if (component_groups) {
            //If there is an item, continue
            if (component_groups[group] !== undefined)
                continue;
        }
        diagnoser.add(`events/${id}/${group}`, `Event is calling component group: ${group}, but the component group was not found`, main_1.DiagnosticSeverity.warning, "behaviorpack.entity.component_group.missing");
    }
}
//# sourceMappingURL=events.js.map