"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.behaviorpack_item_diagnose = void 0;
const Entity_1 = require("../Entity");
const Definitions_1 = require("../../Definitions");
const Types_1 = require("../../../Types");
const bc_minecraft_bedrock_vanilla_data_1 = require("bc-minecraft-bedrock-vanilla-data");
function behaviorpack_item_diagnose(value, diagnoser) {
    let id = value.text;
    //Defined in McProject
    if ((0, Definitions_1.check_definition_value)(diagnoser.project.definitions.item, id, diagnoser))
        return true;
    //If it is an spawn egg, treat it as an entity
    if (id.endsWith("_spawn_egg")) {
        const entity = { offset: value.offset, text: id.slice(0, id.length - 10) };
        return (0, Entity_1.behaviorpack_entityid_diagnose)(entity, diagnoser);
    }
    if (hasAny(id, diagnoser))
        return checkData(value, diagnoser);
    //Missing namespace?
    if (!id.includes(":")) {
        //retry
        id = "minecraft:" + id;
        if (hasAny(id, diagnoser)) {
            value = { offset: value.offset, text: id };
            return checkData(value, diagnoser);
        }
    }
    //Nothing then report error
    diagnoser.add(value, `Cannot find behaviorpack item definition: ${id}`, Types_1.DiagnosticSeverity.error, "behaviorpack.item.missing");
    return false;
}
exports.behaviorpack_item_diagnose = behaviorpack_item_diagnose;
function hasAny(id, diagnoser) {
    const data = diagnoser.context.getCache();
    if ((0, Definitions_1.check_definition_value)(diagnoser.project.definitions.item, id, diagnoser))
        return true;
    if ((0, Definitions_1.check_definition_value)(diagnoser.project.definitions.block, id, diagnoser))
        return true;
    //Project has item Or blocks
    if (data.hasItem(id))
        return true;
    if (data.hasBlock(id))
        return true;
    const edu = (0, Definitions_1.education_enabled)(diagnoser);
    //Vanilla has item
    if (bc_minecraft_bedrock_vanilla_data_1.MinecraftData.BehaviorPack.hasItem(id, edu))
        return true;
    if (bc_minecraft_bedrock_vanilla_data_1.MinecraftData.BehaviorPack.hasBlock(id, edu))
        return true;
    return false;
}
function checkData(value, diagnoser) {
    const edu = (0, Definitions_1.education_enabled)(diagnoser);
    const item = bc_minecraft_bedrock_vanilla_data_1.MinecraftData.BehaviorPack.getItem(value.text, edu);
    if (item && typeof value.data === "number") {
        if (value.data <= item.max_damage) {
            diagnoser.add(value, `Item data is for ${value.text} is 0..${item.max_damage}`, Types_1.DiagnosticSeverity.error, "behaviorpack.item.data");
        }
    }
    return true;
}
//# sourceMappingURL=diagnose.js.map