"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.behaviorpack_loot_table_short_diagnose = exports.behaviorpack_loot_table_diagnose = void 0;
const Definitions_1 = require("../../Definitions");
const Types_1 = require("../../../Types");
const bc_minecraft_bedrock_vanilla_data_1 = require("bc-minecraft-bedrock-vanilla-data");
function behaviorpack_loot_table_diagnose(value, diagnoser) {
    const id = typeof value === "string" ? value : value.text;
    if (has_loot_table(id, diagnoser)) {
        return true;
    }
    //Nothing then report error
    diagnoser.add(value, `Cannot find behaviorpack loot_table definition: ${id}`, Types_1.DiagnosticSeverity.error, "behaviorpack.loot_table.missing");
    return false;
}
exports.behaviorpack_loot_table_diagnose = behaviorpack_loot_table_diagnose;
function behaviorpack_loot_table_short_diagnose(value, diagnoser) {
    let id = typeof value === "string" ? value : value.text;
    //Strip ""
    if (id.startsWith('"')) {
        id = id.slice(1);
    }
    if (id.endsWith('"')) {
        id = id.slice(0, -1);
    }
    if (!id.startsWith("loot_tables/")) {
        id = "loot_tables/" + id;
    }
    if (!id.endsWith(".json")) {
        id = id + ".json";
    }
    if (has_loot_table(id, diagnoser)) {
        return true;
    }
    diagnoser.add(value, `Cannot find behaviorpack loot_table definition: ${id}`, Types_1.DiagnosticSeverity.error, "behaviorpack.loot_table.missing");
    return false;
}
exports.behaviorpack_loot_table_short_diagnose = behaviorpack_loot_table_short_diagnose;
function has_loot_table(id, diagnoser) {
    //Defined in McProject
    if ((0, Definitions_1.check_definition_value)(diagnoser.project.definitions.loot_table, id, diagnoser))
        return true;
    const data = diagnoser.context.getCache();
    //Project has loot_table
    if (data.BehaviorPacks.loot_tables.has(id))
        return true;
    const edu = (0, Definitions_1.education_enabled)(diagnoser);
    //Vanilla has loot_table
    if (bc_minecraft_bedrock_vanilla_data_1.MinecraftData.BehaviorPack.hasLootTable(id, edu))
        return true;
    return false;
}
//# sourceMappingURL=diagnose.js.map