"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.is_animation_or_controller = exports.anim_or_contr = exports.animation_or_controller_diagnose_implementation = void 0;
const Severity_1 = require("../../Types/Severity");
const diagnostics_1 = require("./Animation Controllers/diagnostics");
const diagnostics_2 = require("./Animation/diagnostics");
/**
 * @param id
 * @param user
 * @param ownerType
 * @param diagnoser
 * @returns
 */
function animation_or_controller_diagnose_implementation(id, user, ownerType, diagnoser) {
    switch (is_animation_or_controller(id, diagnoser)) {
        case anim_or_contr.animation:
            return (0, diagnostics_2.animation_diagnose_implementation)(id, user, ownerType, diagnoser);
        case anim_or_contr.controller:
            return (0, diagnostics_1.animation_controller_diagnose_implementation)(id, user, ownerType, diagnoser);
        case anim_or_contr.neither:
            diagnoser.add(id, `Cannot find animation / animation controller: ${id}`, Severity_1.DiagnosticSeverity.error, "behaviorpack.anim_or_controller.missing");
    }
}
exports.animation_or_controller_diagnose_implementation = animation_or_controller_diagnose_implementation;
/** The result of the animation or controller check */
var anim_or_contr;
(function (anim_or_contr) {
    /** the id is an animation */
    anim_or_contr[anim_or_contr["animation"] = 0] = "animation";
    /** the id is an animation controller */
    anim_or_contr[anim_or_contr["controller"] = 1] = "controller";
    /** the id is neither an animation nor an animation controller */
    anim_or_contr[anim_or_contr["neither"] = 2] = "neither";
})(anim_or_contr || (exports.anim_or_contr = anim_or_contr = {}));
/** is an animation or controller.
 * @param id The id of the animation or controller
 * @param diagnoser The diagnostics builder to add the errors to
 * @returns True if animation, false if controller*/
function is_animation_or_controller(id, diagnoser) {
    const cache = diagnoser.context.getCache();
    if (cache.BehaviorPacks.animations.has(id))
        return anim_or_contr.animation;
    if (cache.BehaviorPacks.animation_controllers.has(id))
        return anim_or_contr.controller;
    return anim_or_contr.neither;
}
exports.is_animation_or_controller = is_animation_or_controller;
//# sourceMappingURL=anim%20or%20controller.js.map