"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.check_definition_value = exports.education_enabled = void 0;
const bc_minecraft_project_1 = require("bc-minecraft-project");
const Severity_1 = require("../Types/Severity");
/**Returns if the project has education enabled
 * @param diagnoser The diagnostics builder to add the errors to
 * @returns True if education is enabled, false if not*/
function education_enabled(diagnoser) {
    return diagnoser.project.attributes["education.enable"] === "true";
}
exports.education_enabled = education_enabled;
/**Checks if the Definition has the given value, if it has then return `true`, if it also excluded will report it to the diagnoser.
 * If it is neither in defined or excluded then `false` is return
 * @param container The container to check
 * @param value The value to find
 * @param diagnoser The diagnoser to report to
 * @returns false is not found in either exclusion or definitions*/
function check_definition_value(container, value, diagnoser) {
    if (bc_minecraft_project_1.Definition.is(container)) {
        //Is defined
        if (container.defined.includes(value))
            return true;
        //Is excluded
        if (container.excluded.includes(value)) {
            diagnoser.add(value, "Value has been blacklisted through the project files", Severity_1.DiagnosticSeverity.error, "project.excluded");
            return true;
        }
    }
    return false;
}
exports.check_definition_value = check_definition_value;
//# sourceMappingURL=Definitions.js.map