"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.diagnoseFormatVersion = exports.diagnoseFormatVersionIf = void 0;
const minecraft_1 = require("bc-minecraft-bedrock-types/lib/src/minecraft");
const Types_1 = require("../../Types");
const Lib_1 = require("bc-minecraft-bedrock-vanilla-data/lib/src/Lib");
var FormatVersionContainer;
(function (FormatVersionContainer) {
    function is(value) {
        if (value && typeof value === "object") {
            if (typeof value.format_version === "string")
                return true;
        }
        return false;
    }
    FormatVersionContainer.is = is;
})(FormatVersionContainer || (FormatVersionContainer = {}));
const latestVersion = minecraft_1.FormatVersion.parse(Lib_1.Versions.latest);
function diagnoseFormatVersionIf(data, diagnoser) {
    if (FormatVersionContainer.is(data))
        diagnoseFormatVersion(data, diagnoser);
}
exports.diagnoseFormatVersionIf = diagnoseFormatVersionIf;
function diagnoseFormatVersion(data, diagnoser) {
    var _a;
    const location = `format.version/${(_a = data.format_version) !== null && _a !== void 0 ? _a : ""}`;
    if (typeof data.format_version !== "string") {
        return diagnoser.add("Format version is not a string", location, Types_1.DiagnosticSeverity.error, "minecraft.format_version");
    }
    let v;
    try {
        v = minecraft_1.FormatVersion.parse(data.format_version);
    }
    catch (err) {
        return diagnoser.add("Format version is not a valid number", location, Types_1.DiagnosticSeverity.error, "minecraft.format_version");
    }
    let [latestVersionMajor, latestVersionMinor, latestVersionPatch] = latestVersion;
    let [major, minor, patch] = v;
    // If version is less then latest recommend upgrade
    for (let I = 0; I < 3; I++) {
        if (latestVersion[I] > v[I]) {
            return diagnoser.add(`Format version is out of date, please upgrade to ${Lib_1.Versions.latest}`, location, Types_1.DiagnosticSeverity.warning, "minecraft.format_version");
        }
    }
}
exports.diagnoseFormatVersion = diagnoseFormatVersion;
//# sourceMappingURL=FormatVersion.js.map