"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Json = void 0;
const jsonc_1 = require("jsonc");
const Severity_1 = require("../../Types/Severity");
const Errors_1 = require("./Errors");
var Json;
(function (Json) {
    /**Loads the object and casts it to the specified type, if it fails then undefined is loaded and the error message is send to the diagnoser
     * @param doc The text document to load from
     * @param diagnoser The diagnoser to load from
     * @returns Either the object cast to the specific type, or undefined if failed*/
    function LoadReport(diagnoser) {
        try {
            //get text
            const text = diagnoser.document.getText();
            //get object
            const temp = parse(text);
            // Format version
            // diagnoseFormatVersionIf(temp, diagnoser);
            return temp;
            //cast object
        }
        catch (err) {
            (0, Errors_1.Handle_Json_Error)(err, diagnoser);
        }
        return undefined;
    }
    Json.LoadReport = LoadReport;
    function parse(text) {
        return jsonc_1.jsonc.parse(text, { stripComments: true });
    }
    Json.parse = parse;
    /**
     *
     * @param value
     * @param diagnoser
     * @param type
     * @param code
     * @param checkFn
     * @returns
     */
    function TypeCheck(value, diagnoser, type, code, checkFn) {
        if (checkFn(value)) {
            return true;
        }
        diagnoser.add(0, "Json cannot be casted to: " + type, Severity_1.DiagnosticSeverity.error, code);
        return false;
    }
    Json.TypeCheck = TypeCheck;
})(Json || (exports.Json = Json = {}));
//# sourceMappingURL=Json.js.map