"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.minecraft_language_line_diagnose = exports.minecraft_language_diagnose = void 0;
const Types_1 = require("../../Types");
const bc_minecraft_bedrock_types_1 = require("bc-minecraft-bedrock-types");
function minecraft_language_diagnose(diagnoser) {
    const keys = [];
    let lastOffset = 0;
    const text = diagnoser.document.getText();
    const lines = text.split("\n");
    for (let I = 0; I < lines.length; I++) {
        const line = lines[I].trim();
        const offset = text.indexOf(line, lastOffset);
        minecraft_language_line_diagnose(bc_minecraft_bedrock_types_1.Types.OffsetWord.create(line, offset), keys, diagnoser);
        lastOffset = offset + 1;
    }
}
exports.minecraft_language_diagnose = minecraft_language_diagnose;
/**
 *
 * @param line
 * @param index The line index
 * @param keys
 * @param diagnoser
 * @returns
 */
function minecraft_language_line_diagnose(line, keys, diagnoser) {
    //Find comment on line
    let text = line.text;
    const CommentIndex = text.indexOf("#");
    //If comment has been found
    if (CommentIndex >= 0) {
        //Ensuring that comment has started right '##'
        if (text.substring(CommentIndex, CommentIndex + 2) !== "##") {
            diagnoser.add(line, "A comment is always ##", Types_1.DiagnosticSeverity.error, "minecraft.language.comment.invalid");
        }
        //Check if the comment doesn't start at the start of the line
        if (CommentIndex > 0) {
            //Comments need to be predicated with a tab if they do not start at the beginning of the line
            if (text.charAt(CommentIndex - 1) !== "\t") {
                diagnoser.add(line, "Before a comment must be a tab", Types_1.DiagnosticSeverity.error, "minecraft.language.comment.invalid");
            }
        }
        //Remove comment;
        text = text.substring(0, CommentIndex).trim();
    }
    //If line is empty
    if (text === "" || text === "\r" || text === "\r\n" || text == "") {
        //If the line was an identend comment, it will leave an empty line
        if (CommentIndex > 0) {
            diagnoser.add(line, "A line cannot have an indented comment", Types_1.DiagnosticSeverity.error, "minecraft.language.indentation");
        }
        return;
    }
    //Find end of key
    const Index = text.indexOf("=");
    //If no key definition has been found, it means an invalid line has been given
    if (Index < 0) {
        diagnoser.add(line, "A translation item needs a '=' to separate key and value", Types_1.DiagnosticSeverity.error, "minecraft.language.separator");
    }
    else {
        const Key = text.substring(0, Index);
        const KeyIndex = keys.findIndex((item) => item.text == Key);
        //If the key is found in the existing list of keys, then produce an error
        if (KeyIndex >= 0) {
            const secondKey = keys[KeyIndex];
            diagnoser.add(line, "Duplicate key found", Types_1.DiagnosticSeverity.error, "minecraft.language.duplicate");
            diagnoser.add(secondKey, "Duplicate key found", Types_1.DiagnosticSeverity.error, "minecraft.language.duplicate");
        }
        keys.push(bc_minecraft_bedrock_types_1.Types.OffsetWord.create(Key, line.offset));
        const value = text.substring(Index + 1);
        let offset = line.offset + Index + 1;
        let index;
        if ((index = value.indexOf("\\n")) > -1)
            diagnoser.add(index + offset, "Illegal text, minecraft doesn't accept this unfortunately", Types_1.DiagnosticSeverity.error, "minecraft.language.illegal");
        if ((index = value.indexOf("\\r")) > -1)
            diagnoser.add(index + offset, "Illegal text, minecraft doesn't accept this unfortunately", Types_1.DiagnosticSeverity.error, "minecraft.language.illegal");
        if ((index = value.indexOf("\\t")) > -1)
            diagnoser.add(index + offset, "Illegal text, minecraft doesn't accept this unfortunately", Types_1.DiagnosticSeverity.error, "minecraft.language.illegal");
    }
    //The value needs to be something
    if (Index >= text.length) {
        diagnoser.add(line, "A value must be at least length of 1 or more", Types_1.DiagnosticSeverity.error, "minecraft.language.value");
    }
}
exports.minecraft_language_line_diagnose = minecraft_language_line_diagnose;
//# sourceMappingURL=Language.js.map