"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Attribute = exports.attribute_diagnostics = void 0;
const json_1 = require("bc-minecraft-bedrock-types/lib/src/minecraft/json");
const Types_1 = require("../../../Types");
const General_1 = require("../../General");
const General_2 = require("./General");
const Coordinate_1 = require("./Coordinate");
const Checks_1 = require("./Checks");
const Mode_1 = require("../../Mode");
const Family_1 = require("../Family");
const Tag_1 = require("../Tag");
const Entity_1 = require("../../BehaviorPack/Entity");
const Name_1 = require("../Name");
const Scores_1 = require("./Scores");
const HasItem_1 = require("./HasItem");
const Util_1 = require("./Util");
function float_diagnose(range) {
    return (0, Util_1.must_offset_word)((value, diagnoser) => (0, General_1.general_float_diagnose)(value, diagnoser, range));
}
exports.attribute_diagnostics = {
    c: (0, Util_1.all)(Checks_1.selectorattributes_no_duplicate, (0, Util_1.must_offset_word)(General_1.general_integer_diagnose)),
    dx: (0, Util_1.all)(Checks_1.selectorattributes_no_duplicate, (0, Util_1.must_offset_word)(Coordinate_1.selectorattribute_coordinate)),
    dy: (0, Util_1.all)(Checks_1.selectorattributes_no_duplicate, (0, Util_1.must_offset_word)(Coordinate_1.selectorattribute_coordinate)),
    dz: (0, Util_1.all)(Checks_1.selectorattributes_no_duplicate, (0, Util_1.must_offset_word)(Coordinate_1.selectorattribute_coordinate)),
    family: (0, Util_1.all)(General_2.selectorattribute_duplicate_check, (0, Util_1.must_offset_word)(Family_1.minecraft_family_diagnose)),
    hasitem: (0, Util_1.all)(Checks_1.selectorattributes_no_duplicate, (0, Util_1.forEach)(HasItem_1.minecraft_selector_hasitem_diagnose)),
    l: (0, Util_1.all)(General_2.selectorattribute_duplicate_check, (0, Util_1.must_offset_word)(General_1.general_integer_diagnose)),
    lm: (0, Util_1.all)(General_2.selectorattribute_duplicate_check, (0, Util_1.must_offset_word)(General_1.general_integer_diagnose)),
    m: (0, Util_1.all)(General_2.selectorattribute_one_positive_all_negatives, (0, Util_1.must_offset_word)(Mode_1.mode_gamemode_diagnose)),
    name: (0, Util_1.all)(General_2.selectorattribute_one_positive_all_negatives, (0, Util_1.must_offset_word)(Name_1.minecraft_name_diagnose)),
    r: (0, Util_1.all)(Checks_1.selectorattributes_no_duplicate, (0, Util_1.must_offset_word)(General_1.general_positive_float_diagnose)),
    rm: (0, Util_1.all)(Checks_1.selectorattributes_no_duplicate, (0, Util_1.must_offset_word)(General_1.general_positive_float_diagnose)),
    rx: (0, Util_1.all)(Checks_1.selectorattributes_no_duplicate, float_diagnose({ min: -90, max: 90 })),
    rxm: (0, Util_1.all)(Checks_1.selectorattributes_no_duplicate, float_diagnose({ min: -90, max: 90 })),
    ry: (0, Util_1.all)(Checks_1.selectorattributes_no_duplicate, float_diagnose({ min: -180, max: 180 })),
    rym: (0, Util_1.all)(Checks_1.selectorattributes_no_duplicate, float_diagnose({ min: -180, max: 180 })),
    scores: (0, Util_1.all)(Checks_1.selectorattributes_no_duplicate, (0, Util_1.forEach)(Scores_1.selector_scores_diagnose)),
    tag: (0, Util_1.all)(General_2.selectorattribute_duplicate_check, (0, Util_1.must_offset_word)(Tag_1.minecraft_tag_diagnose)),
    type: (0, Util_1.all)(General_2.selectorattribute_one_positive_all_negatives, (0, Util_1.must_offset_word)(Entity_1.behaviorpack_entityid_diagnose)),
    x: (0, Util_1.all)(Checks_1.selectorattributes_no_duplicate, (0, Util_1.must_offset_word)(Coordinate_1.selectorattribute_coordinate)),
    y: (0, Util_1.all)(Checks_1.selectorattributes_no_duplicate, (0, Util_1.must_offset_word)(Coordinate_1.selectorattribute_coordinate)),
    z: (0, Util_1.all)(Checks_1.selectorattributes_no_duplicate, (0, Util_1.must_offset_word)(Coordinate_1.selectorattribute_coordinate)),
};
var Attribute;
(function (Attribute) {
    function diagnose(attribute, attributes, sel, diagnoser) {
        const fn = exports.attribute_diagnostics[attribute];
        if (typeof fn === "function") {
            return fn(attributes, sel, diagnoser);
        }
        return defaultAttribute(attribute, attributes, sel, diagnoser);
    }
    Attribute.diagnose = diagnose;
})(Attribute || (exports.Attribute = Attribute = {}));
function defaultAttribute(attribute, attributes, sel, diagnoser) {
    const msg = `Unknown attribute: ${attribute}`;
    attributes.forEach((a) => {
        diagnoser.add(json_1.CompactJson.toOffsetWord(a), msg, Types_1.DiagnosticSeverity.error, "minecraft.selector.attribute.unknown");
    });
    return false;
}
//# sourceMappingURL=Attributes.js.map