"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.minecraft_selector_hasitem_diagnose = exports.attribute_hasitem_diagnostics = void 0;
const json_1 = require("bc-minecraft-bedrock-types/lib/src/minecraft/json");
const Types_1 = require("../../../Types");
const diagnose_1 = require("../../BehaviorPack/Item/diagnose");
const Integer_1 = require("../../General/Integer");
const diagnose_2 = require("../../Mode/diagnose");
const Util_1 = require("./Util");
const General_1 = require("./General");
const Checks_1 = require("./Checks");
const General_2 = require("../../General");
function integer_diagnose(range) {
    return (0, Util_1.must_offset_word)((value, diagnoser) => (0, Integer_1.general_integer_diagnose)(value, diagnoser, range));
}
function range_integer_diagnose(range) {
    return (0, Util_1.must_offset_word)((value, diagnoser) => (0, General_2.general_range_integer_diagnose)(value, diagnoser, range));
}
exports.attribute_hasitem_diagnostics = {
    item: (0, Util_1.all)(Checks_1.selectorattributes_no_duplicate, General_1.selectorattribute_no_negatives, (0, Util_1.must_offset_word)(diagnose_1.behaviorpack_item_diagnose)),
    //Has extra checks down below
    data: (0, Util_1.all)(Checks_1.selectorattributes_no_duplicate, General_1.selectorattribute_no_negatives, integer_diagnose({ min: -1, max: 32767 })),
    quantity: (0, Util_1.all)(General_1.selectorattribute_one_positive_all_negatives, range_integer_diagnose({ min: 0, max: 32767 })),
    location: (0, Util_1.all)(Checks_1.selectorattributes_no_duplicate, General_1.selectorattribute_one_positive_all_negatives, (0, Util_1.must_offset_word)(diagnose_2.mode_slottype_diagnose)),
    //Has extra checks down below
    slot: (0, Util_1.all)(Checks_1.selectorattributes_no_duplicate, General_1.selectorattribute_one_positive_all_negatives, integer_diagnose({ min: 0, max: 53 })),
};
/**
 * Diagnoses the hasitem selector attribute
 * @param attr
 * @param sel
 * @param diagnoser
 */
function minecraft_selector_hasitem_diagnose(attr, sel, diagnoser) {
    if (json_1.CompactJson.isString(attr)) {
        diagnoser.add(json_1.CompactJson.toOffsetWord(attr), "The hasitem attribute needs to be either a array or object", Types_1.DiagnosticSeverity.error, "minecraft.selector.hasitem.type");
        return false;
    }
    if (json_1.CompactJson.isObject(attr)) {
        return diagnose_hasitem_object(attr, sel, diagnoser);
    }
    let result = true;
    if (json_1.CompactJson.isArray(attr)) {
        for (let a of attr.value) {
            //Sub items need to be a object
            if (json_1.CompactJson.isObject(a)) {
                result =
                    diagnose_hasitem_object(json_1.CompactJson.toKeyed(a, "hasitem"), sel, diagnoser) && result;
            }
            else {
                diagnoser.add(json_1.CompactJson.toOffsetWord(a), "Expected a object", Types_1.DiagnosticSeverity.error, "minecraft.selector.hasitem.type");
                result = false;
            }
        }
    }
    return result;
}
exports.minecraft_selector_hasitem_diagnose = minecraft_selector_hasitem_diagnose;
function diagnose_hasitem_object(attr, sel, diagnoser) {
    let result = true;
    const reader = new json_1.CompactJsonReader(attr);
    //Hasitem needs to contain the item attribute
    if (!reader.contains("item")) {
        diagnoser.add(json_1.CompactJson.toOffsetWord(attr), "Missing item selector attribute", Types_1.DiagnosticSeverity.error, "minecraft.selector.hasitem.item.missing");
        result = false;
    }
    const names = reader.names();
    for (const name of names) {
        const checks = exports.attribute_hasitem_diagnostics[name];
        const attrs = reader.get(name);
        if (checks) {
            result = checks(attrs, sel, diagnoser) && result;
        }
        else {
            result = defaultAttribute(name, attrs, sel, diagnoser) && result;
        }
        //If still good we check perform additional checks
        if (result) {
            switch (name) {
                case "data":
                    result = diagnose_hasitem_data(attrs, reader, diagnoser) && result;
                    break;
                case "slot":
                    result = diagnose_hasitem_slot(attrs, reader, diagnoser) && result;
                    break;
            }
        }
    }
    return result;
}
function defaultAttribute(attribute, attributes, sel, diagnoser) {
    const msg = `Unknown attribute: ${attribute}`;
    attributes.forEach((a) => {
        diagnoser.add(json_1.CompactJson.toOffsetWord(a), msg, Types_1.DiagnosticSeverity.error, "minecraft.selector.hasitem.attribute.invalid");
    });
    return false;
}
function diagnose_hasitem_data(attrs, reader, diagnoser) {
    let result = true;
    const item = reader.get("item");
    if (item.length !== 1) {
        return false;
    }
    const itemWord = json_1.CompactJson.valueToOffsetWord(item[0]);
    attrs.forEach((a) => {
        if (json_1.CompactJson.isString(a)) {
            itemWord.data = parseInt(a.value);
            result = (0, diagnose_1.behaviorpack_item_diagnose)(itemWord, diagnoser) && result;
        }
    });
    return result;
}
function diagnose_hasitem_slot(attrs, reader, diagnoser) {
    let result = true;
    const location = reader.get("location");
    if (location.length !== 1) {
        return false;
    }
    const locationWord = json_1.CompactJson.valueToOffsetWord(location[0]);
    attrs.forEach((a) => {
        if (json_1.CompactJson.isString(a)) {
            result = (0, diagnose_2.mode_slotid_diagnose)(locationWord, a.value, diagnoser) && result;
        }
    });
    return result;
}
//# sourceMappingURL=HasItem.js.map