"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.diagnoser_molang_syntax = exports.diagnose_molang_syntax_expression = void 0;
const bc_minecraft_molang_1 = require("bc-minecraft-molang");
const Types_1 = require("../../Types");
const Molang_1 = require("bc-minecraft-molang/lib/src/Molang");
/**
 *
 * @param molang
 * @param diagnoser
 */
function diagnose_molang_syntax_expression(molang, diagnoser) {
    if ((0, Molang_1.IsMolangType)(molang) === Molang_1.MolangType.unknown) {
        return;
    }
    const complex = molang.includes(";") || has_assign_operator(molang);
    if (complex && !molang.endsWith(";")) {
        diagnoser.add(molang, `Molang expression is complex and should end with a ';'`, Types_1.DiagnosticSeverity.error, "molang.syntax.missing_semicolon");
    }
}
exports.diagnose_molang_syntax_expression = diagnose_molang_syntax_expression;
/**
 *
 * @param data
 * @param diagnoser
 */
function diagnoser_molang_syntax(data, diagnoser) {
    bc_minecraft_molang_1.Molang.Traverse(data, (molang, type) => {
        // Skip commands
        if (type === Molang_1.MolangType.command) {
            return;
        }
        diagnose_molang_syntax_expression(molang, diagnoser);
    });
}
exports.diagnoser_molang_syntax = diagnoser_molang_syntax;
function has_assign_operator(text) {
    // Using regex, Check for = but ensure its not = or <= or >=
    return /[^<>!+\-*\/=]=[^<>=]/.test(text);
}
//# sourceMappingURL=syntax.js.map