"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.has_animation = exports.animation_diagnose_implementation = void 0;
const Severity_1 = require("../../../Types/Severity");
const Definitions_1 = require("../../Definitions");
const bc_minecraft_bedrock_vanilla_data_1 = require("bc-minecraft-bedrock-vanilla-data");
const Molang_1 = require("../../Molang");
/**
 *
 * @param id
 * @param data
 * @param diagnoser
 */
function animation_diagnose_implementation(anim_id, user, ownerType, diagnoser, particles, sounds) {
    if (!has_animation(anim_id, diagnoser))
        return;
    //Project has animation
    const anim = diagnoser.context.getCache().ResourcePacks.animations.get(anim_id);
    if (!anim)
        return;
    (0, Molang_1.diagnose_molang_implementation)(anim, user, ownerType, diagnoser);
    //Particle check
    anim.particles.using.forEach((particle) => {
        if (particles && particles[particle] !== undefined)
            return;
        diagnoser.add(`animations/${anim_id}`, `Animation: ${anim_id} uses particle: '${particle}', but no definition has been found`, Severity_1.DiagnosticSeverity.warning, "resourcepack.particle.missing");
    });
    //Sound check
    anim.sounds.using.forEach((sound) => {
        if (sounds && sounds[sound] !== undefined)
            return;
        diagnoser.add(`animations/${anim_id}`, `Animation: ${anim_id} uses sound: '${sound}', but no definition has been found`, Severity_1.DiagnosticSeverity.warning, "resourcepack.sound.missing");
    });
}
exports.animation_diagnose_implementation = animation_diagnose_implementation;
/**
 *
 * @param id
 * @param diagnoser
 * @returns
 */
function has_animation(id, diagnoser) {
    const cache = diagnoser.context.getCache();
    //Project has render controller
    if (cache.ResourcePacks.animations.has(id))
        return true;
    const edu = (0, Definitions_1.education_enabled)(diagnoser);
    //Vanilla has render controller
    if (bc_minecraft_bedrock_vanilla_data_1.MinecraftData.ResourcePack.hasAnimation(id, edu))
        return true;
    //Nothing then report error
    diagnoser.add(`"${id}"`, `Cannot find resourcepack animation: ${id}`, Severity_1.DiagnosticSeverity.error, "resourcepack.animation.missing");
    return false;
}
exports.has_animation = has_animation;
//# sourceMappingURL=diagnostics.js.map