"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Diagnose = void 0;
const bc_minecraft_bedrock_project_1 = require("bc-minecraft-bedrock-project");
const Types_1 = require("../../../Types");
const diagnostics_1 = require("../../Molang/diagnostics");
const Json_1 = require("../../Json/Json");
const Model_1 = require("../Model");
/**Diagnoses the given document as an animation
 * @param doc The text document to diagnose
 * @param diagnoser The diagnoser builder to receive the errors*/
function Diagnose(diagnoser) {
    //Check molang
    (0, diagnostics_1.diagnose_molang)(diagnoser.document.getText(), "Animations", diagnoser);
    const anims = Json_1.Json.LoadReport(diagnoser);
    if (!bc_minecraft_bedrock_project_1.Internal.ResourcePack.Animations.is(anims))
        return;
    const bones = [];
    bc_minecraft_bedrock_project_1.SMap.forEach(anims.animations, (anim, anim_id) => {
        const length = anim.animation_length;
        bc_minecraft_bedrock_project_1.SMap.forEach(anim.bones, (bone, bone_id) => {
            bones.push({ bone_id, parent_id: anim_id });
            if (typeof length === "number") {
                check_bone_time(`${anim_id}/${bone_id}`, bone, length, diagnoser);
            }
        });
    });
    (0, Model_1.checkBonesExists)(bones, diagnoser);
}
exports.Diagnose = Diagnose;
function check_bone_time(parentid, bone, animation_length, diagnoser) {
    check_bone_property_time(parentid, bone.position, animation_length, diagnoser);
    check_bone_property_time(parentid, bone.rotation, animation_length, diagnoser);
    check_bone_property_time(parentid, bone.scale, animation_length, diagnoser);
}
function check_bone_property_time(parentid, property, animation_length, diagnoser) {
    if (typeof property !== "object")
        return;
    if (Array.isArray(property))
        return;
    for (const k of Object.keys(property)) {
        //Parse as float
        try {
            const time = parseFloat(k);
            if (isNaN(time)) {
                diagnoser.add(k, `Failed to parse time value: ${k}`, Types_1.DiagnosticSeverity.error, "general.float.invalid");
            }
            else if (time > animation_length) {
                diagnoser.add(`${parentid}/${k}`, `Time value of bone ${k} is greater than the animation length: ${animation_length}`, Types_1.DiagnosticSeverity.error, "resourcepack.animation.time.exceeds");
            }
        }
        catch (err) {
            diagnoser.add(k, `Failed to parse time value: ${k}\n${err}`, Types_1.DiagnosticSeverity.error, "general.float.invalid");
        }
    }
}
//# sourceMappingURL=entry.js.map