"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Diagnose = void 0;
const bc_minecraft_bedrock_project_1 = require("bc-minecraft-bedrock-project");
const Json_1 = require("../../Json/Json");
const diagnostics_1 = require("../Animation Controllers/diagnostics");
const anim_or_controller_1 = require("../anim or controller");
const bc_minecraft_molang_1 = require("bc-minecraft-molang");
const bc_minecraft_bedrock_types_1 = require("bc-minecraft-bedrock-types");
const diagnostics_2 = require("../../Molang/diagnostics");
const diagnostics_3 = require("../Render Controller/diagnostics");
const diagnose_1 = require("../Model/diagnose");
const entry_1 = require("../Texture Atlas/entry");
const diagnose_2 = require("../Particle/diagnose");
const Script_1 = require("../../Minecraft/Script");
const diagnostics_4 = require("../Sounds/diagnostics");
const usage_1 = require("../Animation/usage");
/**Diagnoses the given document as an attachable
 * @param doc The text document to diagnose
 * @param diagnoser The diagnoser builder to receive the errors*/
function Diagnose(diagnoser) {
    var _a, _b;
    //Check molang
    (0, diagnostics_2.diagnose_molang)(diagnoser.document.getText(), "Items", diagnoser);
    //Load attacble
    const attacble = Json_1.Json.LoadReport(diagnoser);
    if (!bc_minecraft_bedrock_project_1.Internal.ResourcePack.Attachable.is(attacble))
        return;
    const description = attacble["minecraft:attachable"].description;
    const attachableGathered = bc_minecraft_bedrock_project_1.ResourcePack.Attachable.Process(diagnoser.document);
    if (!attachableGathered)
        return;
    if (!attachableGathered.molang)
        attachableGathered.molang = bc_minecraft_molang_1.Molang.MolangFullSet.harvest(diagnoser.document.getText());
    //#region animations
    //Check animations / animation controllers
    bc_minecraft_bedrock_types_1.Types.Definition.forEach(description.animations, (reference, anim_id) => (0, anim_or_controller_1.animation_or_controller_diagnose_implementation)(anim_id, attachableGathered, "Attachables", diagnoser, description.particle_effects, description.sound_effects));
    //Check used animations
    (0, usage_1.resourcepack_animation_used)(description.animations, diagnoser, description.scripts);
    //#endregion
    //Check animation controllers
    (_a = description.animation_controllers) === null || _a === void 0 ? void 0 : _a.forEach((controller) => {
        const temp = flatten(controller);
        if (temp)
            (0, diagnostics_1.animation_controller_diagnose_implementation)(temp, attachableGathered, "Attachables", diagnoser);
    });
    //Check render controllers
    (_b = description.render_controllers) === null || _b === void 0 ? void 0 : _b.forEach((controller) => {
        const temp = getKey(controller);
        if (temp)
            (0, diagnostics_3.render_controller_diagnose_implementation)(temp, attachableGathered, "Attachables", diagnoser);
    });
    //Check models
    bc_minecraft_bedrock_types_1.Types.Definition.forEach(description.geometry, (ref, modelId) => (0, diagnose_1.resourcepack_has_model)(modelId, diagnoser));
    //check particles
    bc_minecraft_bedrock_types_1.Types.Definition.forEach(description.particle_effects, (ref, part_id) => (0, diagnose_2.resourcepack_particle_diagnose)(part_id, diagnoser));
    //Get pack
    const pack = diagnoser.context.getCache().ResourcePacks.get(diagnoser.document.uri);
    if (pack === undefined)
        return;
    const rp_files = diagnoser.context
        .getFiles(pack.folder, ["**/textures/**/*.{tga,png,jpg,jpeg}"], pack.context.ignores)
        .map((item) => item.replace(/\\/gi, "/"));
    //Check if attachable has textures defined
    bc_minecraft_bedrock_types_1.Types.Definition.forEach(description.textures, (ref, id) => {
        (0, entry_1.texture_files_diagnose)(description.identifier, id, rp_files, diagnoser);
    });
    //Check if attachable has sounds defined
    (0, diagnostics_4.diagnose_resourcepack_sounds)(description.sound_effects, diagnoser);
    //Script check
    if (description.scripts)
        (0, Script_1.diagnose_script)(diagnoser, description.scripts, description.animations);
}
exports.Diagnose = Diagnose;
function flatten(data) {
    if (typeof data === "string")
        return data;
    const key = Object.getOwnPropertyNames(data)[0];
    if (key)
        return data[key];
    return undefined;
}
function getKey(data) {
    if (typeof data === "string")
        return data;
    return Object.getOwnPropertyNames(data)[0];
}
//# sourceMappingURL=entry.js.map