"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Diagnose = void 0;
const Severity_1 = require("../../../Types/Severity");
const Json_1 = require("../../Json/Json");
/**Diagnoses the given document as a block.json
 * @param doc The text document to diagnose
 * @param diagnoser The diagnoser builder to receive the errors*/
function Diagnose(diagnoser) {
    const blocks = Json_1.Json.LoadReport(diagnoser);
    if (!Json_1.Json.TypeCheck(blocks, diagnoser, "blocks.json", "resourcepack.blocks.invalid", is))
        return;
    const keys = Object.keys(blocks);
    const rp = diagnoser.context.getCache().ResourcePacks;
    for (let I = 0; I < keys.length; I++) {
        const key = keys[I];
        if (key === "format_version")
            continue;
        const block = blocks[keys[I]];
        if (typeof block === "object") {
            const texture = block.textures;
            if (typeof texture === "string") {
                hasDefinition(key, texture, rp, diagnoser);
            }
            else if (texture) {
                if (texture.down)
                    hasDefinition(key, texture.down, rp, diagnoser);
                if (texture.up)
                    hasDefinition(key, texture.up, rp, diagnoser);
                if (texture.side)
                    hasDefinition(key, texture.side, rp, diagnoser);
                if (texture.north)
                    hasDefinition(key, texture.north, rp, diagnoser);
                if (texture.south)
                    hasDefinition(key, texture.south, rp, diagnoser);
                if (texture.west)
                    hasDefinition(key, texture.west, rp, diagnoser);
                if (texture.east)
                    hasDefinition(key, texture.east, rp, diagnoser);
            }
        }
    }
}
exports.Diagnose = Diagnose;
function hasDefinition(block, value, rp, diagnoser) {
    if (rp.textures.has(value))
        return;
    diagnoser.add(`${block}/${value}`, "The texture is not defined in the terrain_texture.json", Severity_1.DiagnosticSeverity.error, "resourcepack.texture.undefined");
}
function is(value) {
    if (typeof value === "object") {
        return true;
    }
    return false;
}
//# sourceMappingURL=entry.js.map